'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var state = require('@codemirror/state');
var view = require('@codemirror/view');

const EditorTools = (view) => {
    const getDoc = () => {
        var _a, _b;
        if ((_b = (_a = view === null || view === void 0 ? void 0 : view.state) === null || _a === void 0 ? void 0 : _a.doc) === null || _b === void 0 ? void 0 : _b.toString()) {
            return view.state.doc.toString();
        }
        return " ";
    };
    const setDoc = (newDoc) => {
        var _a, _b;
        if (newDoc !== getDoc() && ((_b = (_a = view === null || view === void 0 ? void 0 : view.state) === null || _a === void 0 ? void 0 : _a.doc) === null || _b === void 0 ? void 0 : _b.toString)) {
            view.dispatch({
                changes: {
                    from: 0,
                    to: view.state.doc.length,
                    insert: newDoc
                }
            });
        }
    };
    const focus = () => view.focus();
    return {
        getDoc,
        setDoc,
        focus
    };
};

var __rest = (undefined && undefined.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
const createEditorState = (_a) => {
    var { onUpdate, onChange, onFocus, onBlur } = _a, config = __rest(_a, ["onUpdate", "onChange", "onFocus", "onBlur"]);
    let state$1 = state.EditorState.create({
        doc: config.doc,
        selection: config.selection,
        extensions: [
            ...(Array.isArray(config.extensions) ? config.extensions : [config.extensions]),
            view.EditorView.updateListener.of((viewUpdate) => {
                onUpdate(viewUpdate);
                // state change on onchange action
                if (viewUpdate.docChanged) {
                    onChange(viewUpdate.state.doc.toString(), viewUpdate);
                }
                // state change on focus action
                if (viewUpdate.focusChanged) {
                    viewUpdate.view.hasFocus ? onFocus(viewUpdate) : onBlur(viewUpdate);
                }
            })
        ]
    });
    return state$1;
};
/**
 *
 * @param config
 * @returns
 */
const createEditorView = (config) => {
    let view$1 = new view.EditorView(Object.assign({}, config));
    return view$1;
};
/**
 * Destroy EdiorView.
 *
 * @param view
 * @returns
 */
const destroyEditorView = (view) => {
    return view.destroy();
};

exports.EditorTools = EditorTools;
exports.createEditorState = createEditorState;
exports.createEditorView = createEditorView;
exports.destroyEditorView = destroyEditorView;
